$ifndef _ADF4351_INC_
$define _ADF4351_INC_
'
' Interface with an ADF4351 based 35MHz to 4.4GHz PLL synthesiser module.
' Written by Les Johnson for the Proton and Proton24 BASIC compilers
'
' My special thanks to Charlie for sending me the synthesiser board.
'
' Setup the default pins to use for the interface
'
$ifndef ADF_DAT_Pin                             ' Has the DAT pin been setup in the main program?
    $define ADF_DAT_Pin PORTB.0                 ' No. So connect to the ADF4351 Dat pin
$endif
$ifndef ADF_CLK_Pin                             ' Has the CLK pin been setup in the main program?
    $define ADF_CLK_Pin PORTB.1                 ' No. So connect to the ADF4351 Clock (CLK) pin
$endif
$ifndef ADF_LD_Pin                              ' Has the LD pin been setup in the main program?
    $define ADF_LD_Pin  PORTB.2                 ' No. So connect from the ADF4351 Lock Detect (LD) pin
$endif
$ifndef ADF_LE_Pin                              ' Has the LE pin been setup in the main program?
    $define ADF_LE_Pin  PORTB.3                 ' No. So connect to the ADF4351 LE pin
$endif

$ifndef True
    $define True 1
$endif
$ifndef False
    $define False 0
$endif
'
' Create some variables
'
    Dim ADF_bFlags      As Byte System          ' Holds flag bits
    Dim ADF_tLock       As ADF_bFlags.0         ' True when the ADF4351 PLL locks
    Dim ADT_tTimedOut   As ADF_bFlags.1         ' True if the lock times out

    Dim ADF_bBitCount   As Byte System          ' Bit counter for the SPI interface
    Dim ADF_bRegIndex   As Byte                 ' ADF4351 Register Counter
    Dim ADF_bRFODivider As Byte                 ' RF output divider value (1 - 64)
    Dim ADF_dOutData    As Dword System         ' Date to send to the ADF4351 registers
    Dim ADF_dIntDiv     As Dword                ' Integer division factor
    Dim ADF_dFracDiv    As Dword                ' Numerator of the fractional division
    Dim ADF_wLockTo     As Word                 ' Hols the timeout value when waiting for a lock

    Dim ADF_dRegs[6] As Dword                   ' Array for ADF4351 control registers
$if _core = 24 Or _core = 33                    ' Is the device a 16-bit core type?
    Dim ADF_dReg0 As ADF_dRegs_0                ' Register 0
    Dim ADF_dReg1 As ADF_dRegs_1                ' Register 1
    Dim ADF_dReg2 As ADF_dRegs_2                ' Register 2
    Dim ADF_dReg3 As ADF_dRegs_3                ' Register 3
    Dim ADF_dReg4 As ADF_dRegs_4                ' Register 4
    Dim ADF_dReg5 As ADF_dRegs_5                ' Register 5
$else                                           ' Otherwise... The device is an 8-bit type
    Dim ADF_dReg0 As ADF_dRegs#0                ' Register 0
    Dim ADF_dReg1 As ADF_dRegs#1                ' Register 1
    Dim ADF_dReg2 As ADF_dRegs#2                ' Register 2
    Dim ADF_dReg3 As ADF_dRegs#3                ' Register 3
    Dim ADF_dReg4 As ADF_dRegs#4                ' Register 4
    Dim ADF_dReg5 As ADF_dRegs#5                ' Register 5
$endif
    Dim ADF_fFrequency As Float                 ' Holds the output frequency required (in MHz)
'
' Create some constants
'
$define cADF_RefFrequency 25.0                  ' Reference frequency input (in MHz)
$define cADF_OutRes 0.01                        ' Output frequency resolution (10KHz)

    Symbol cADF_FracMod = (cADF_RefFrequency / cADF_OutRes) ' Fractional modulus (2 - 4095)
    Symbol cADF_FracModSh3 = (cADF_FracMod << 3) ' Bit position calculation

$define ADF_cSPIDelay() DelayUS 30              ' The small delay used in the SPI interface

'-----------------------------------------------------------------
$if _core <> 24 And _core <> 33                 ' Is the device a 16-bit core type?
    GoTo ADF_Main                               ' No. So jump to the main section
$endif
'-----------------------------------------------------------------
' Wait until the ADF4351 PLL is in locked state
' Input     : pTimeout holds the timeout value (in approx milliseconds)
' Output    : ADF_tLock true if the chip is locked
'           : ADT_tTimedOut true if a lock has not happened and the loop has timed out
' Notes     : None
'
$define ADF_WaitForPLL_Lock(pTimeout) '
    ADF_wLockTo = pTimeout            '
    _ADF_WaitForLock()

Sub _ADF_WaitForLock()
    ADF_tLock = False                           ' Default to no lock
    ADT_tTimedOut = False                       ' Default to no timeout
    Repeat                                      ' Create a loop to wait for the PLL lock
        If ADF_wLockTo = 0 Then                 ' Has the timeout counter reached 0?
            ADT_tTimedOut = True                ' Yes. So signal a timeout
            Return                              ' And exit the subroutine
        EndIf
        DelayMS 1                               ' Delay for 1 ms
        Dec ADF_wLockTo                         ' Decrement the timeout counter
    Until ADF_LD_Pin = 1                        ' Wait until the LD pin is high
    ADF_tLock = True                            ' If the program has gotten here, a lock has occured
EndSub

'------------------------------------------------------------------
' SPI mode 00 interface to the ADF4351
' Input     : ADF_dOutData
' Output    : None
' Notes     : None
'
$define ADF_WriteSPI(pData) '
    ADF_dOutData = pData    '
    _ADF_WriteSPI()

Sub _ADF_WriteSPI()
    For ADF_bBitCount = 31 To 0 Step -1         ' Create a loop for the 32-bits of data to send
        Clear ADF_DAT_Pin                       ' Default to a clear bit
        If ADF_dOutData.31 = 1 Then             ' Is the bit of the value to send a 1?
            Set ADF_DAT_Pin                     ' Yes. So place the outgoing bit onto the ADF_DAT_Pin
        EndIf
        ADF_cSPIDelay()                         ' A small delay
        ADF_dOutData = ADF_dOutData << 1        ' Shift the next bit into MSB
        Set ADF_CLK_Pin                         ' Set the SCK pin high
        ADF_cSPIDelay()                         ' A small delay
        Clear ADF_CLK_Pin                       ' Pull the SCK pin low
    Next
EndSub

'-----------------------------------------------------------------
' Send the values to the six ADF4351 registers
' Input     : Array ADF_dRegs holds the 6 register values
' Output    : ADF_tLock True if the chip is locked
'           : ADT_tTimedOut true if a lock has not happened and the loop has timed out
' Notes     : None
'
Sub ADF_WriteRegData()
    For ADF_bRegIndex = 5 To 0 Step -1          ' Create a loop for all 6 registers
        Clear ADF_LE_Pin                        ' Pull the LE pin low to start a transaction
        ADF_WriteSPI(ADF_dRegs[ADF_bRegIndex])  ' Send the 32-bit register data
        Set ADF_LE_Pin                          ' Set the LE pin high to load it into a register
        DelayUS 100                             ' Delay for 100us to make sure data is latched
        Clear ADF_LE_Pin                        ' Pull the LE pin low
    Next
    ADF_WaitForPLL_Lock(1000)                   ' Wait until the chip locks
EndSub

'-----------------------------------------------------------------
' Calculate ADF4351 register settings for a new frequency
' Input     : ADF_fFrequency holds the frequency (in MHz)
' Output    : ADF_dReg0 to ADF_dReg5 hold the register values
'           : ADT_tTimedOut true if a lock has not happened and the loop has timed out
' Notes     : None
'
$define ADF_SetFreq(pFreq) '
    ADF_fFrequency = pFreq '
    _ADF_SetFreq()

Sub _ADF_SetFreq()
'
' Setup ADF_dReg4 (register 4)
'
    ADF_dReg4 = $0000803C                       ' Setup the default value for register 4
    Select ADF_fFrequency                       ' What frequency range is required?
        Case 2200.0 To 4400.0                   ' Is it 2200MHz to 4400MHz?
            ADF_bRFODivider = 1                 ' Yes. So set the divider value to 1
            ADF_dReg4.Byte2 = $8C               ' Alter byte 2 of register 4
        Case 1100.0 To 2200.0                   ' Is it 1100MHz to 2200MHz?
            ADF_bRFODivider = 2                 ' Yes. So set the divider value to 2
            ADF_dReg4.Byte2 = $9C               ' Alter byte 2 of register 4
        Case 550.0 To 1100.0                    ' Is it 550MHz to 110MHz?
            ADF_bRFODivider = 4                 ' Yes. So set the divider value to 4
            ADF_dReg4.Byte2 = $AC               ' Alter byte 2 of register 4
        Case 275.0 To 550.0                     ' Is it 275MHz to 550MHz?
            ADF_bRFODivider = 8                 ' Yes. So set the divider value to 8
            ADF_dReg4.Byte2 = $BC               ' Alter byte 2 of register 4
        Case 137.5 To 275.0                     ' Is it 137.5MHz to 275MHz?
            ADF_bRFODivider = 16                ' Yes. So set the divider value to 16
            ADF_dReg4.Byte2 = $CC               ' Alter byte 2 of register 4
        Case 68.75 To 137.5                     ' Is it 68.75MHz to 137.5MHz?
            ADF_bRFODivider = 32                ' Yes. So set the divider value to 32
            ADF_dReg4.Byte2 = $DC               ' Alter byte 2 of register 4
        Case 35.0 To 68.75                      ' Is it 35MHz to 68.75MHz?
            ADF_bRFODivider = 64                ' Yes. So set the divider value to 64
            ADF_dReg4.Byte2 = $EC               ' Alter byte 2 of register 4
        Case Else                               ' Otherwise. The frequency is not recognised. So...
            Return                              ' Exit the routine without making changes
    EndSelect
'
' Calculate the values for ADF_dIntDiv and ADF_dFracDiv
'
    ADF_dIntDiv = (ADF_fFrequency * ADF_bRFODivider) / cADF_RefFrequency
    ADF_dFracDiv = (((ADF_fFrequency * ADF_bRFODivider) / cADF_RefFrequency) - ADF_dIntDiv) * cADF_FracMod
'
' Setup ADF_dReg0 (register 0)
'
    ADF_dReg0 = ADF_dIntDiv << 15                   ' Move ADF_dIntDiv into bits 15 - 30
    ADF_dReg0 = ADF_dReg0 + (ADF_dFracDiv << 3)     ' Insert ADF_dFracDiv to bits 3 - 14
'
' Setup ADF_dReg1 (register 1)
'
    ADF_dReg1 = $08008000                           ' Set the default value
    ADF_dReg1 = ADF_dReg1 + cADF_FracModSh3         ' Move cADF_FracMod into bits 3 - 14
    Inc ADF_dReg1                                   ' Add 1 for reg addr
'
' Setup ADF_dReg2, ADF_dReg3 and ADF_dReg5 (registers 2, 3, and 5)
'
    ADF_dReg2 = $00004E42                           ' MUX output disabled
    ADF_dReg3 = $000004B3                           ' PD polarity positive, clk divider off
    ADF_dReg5 = $00580005                           ' LD pin set for dig ADF_tLock detect

    ADF_WriteRegData()                              ' Write the new register values to the ADF4351 chip
EndSub

'-----------------------------------------------------------------
' Setup the program and the ADF4351 SPI interface
' Input     : None
' Output    : None
' Notes     : None
'
Sub ADF_Setup()
    DelayMS 500                                 ' Wait for the ADF4351 board to settle after power up

    Low ADF_DAT_Pin                             ' Pull the DAT pin to low
    Low ADF_CLK_Pin                             ' Pull the CLK pin to low
    High ADF_LE_Pin                             ' Set the LE pin to high
    Input ADF_LD_Pin                            ' Make the LD pin an input

    ADT_tTimedOut = False                       ' Locked time out flag (true if timed out)
    ADF_tLock = False                           ' ADF4351 PLL Lock status (true if locked)
    Clear ADF_dRegs                             ' Clear the registers array
EndSub

'---------------------------------------------------------------------------
' The Starting part of the library
'
ADF_Main:
    ADF_Setup()                                 ' Setup the program and the SPI interface

$endif      ' _ADF4351_INC_

