'
' Interface with an ADF4351 based 35MHz to 4.4GHz PLL synthesiser module.
' Written by Les Johnson for the Proton BASIC compiler
'
    Include "Amicus18.inc"                      ' We'll use the Amicus18 development board, which is a PIC18F25K20 running at 64MHz
'
' Setup the pins to use for the interface (These must be placed before the ADF4351 library is included in the program)
'
    $define ADF_DAT_Pin PORTB.0                 ' Connect to the ADF4351 DAT pin
    $define ADF_CLK_Pin PORTB.1                 ' Connect to the ADF4351 CLK pin
    $define ADF_LD_Pin  PORTB.2                 ' Connect from the ADF4351 Lock Detect (LD) pin
    $define ADF_LE_Pin  PORTB.3                 ' Connect to the ADF4351 LE pin

    Include "ADF4351.inc"                       ' Load the ADF4351 interface routines into the program

    Dim Frequency As Float                      ' Holds the frequency for the demo code

'---------------------------------------------------------------------------
' Sweep the frequency from 88.0 MHz to 108.0 MHz in 10KHz steps
'
Main:
    Do                                          ' Create an infinite loop
        For Frequency = 88.0 To 108.0 Step 0.01 ' Create a loop for the frequency scan
            Do                                  ' Create a loop
                ADF_SetFreq(Frequency)          ' Alter the frequency of the ADF4351 board
                If ADT_tTimedOut = True Then    ' Did the PLL lock time out, while waiting?
                    PLL_TimeOut()               ' Yes. So send a message to the serial terminal
                EndIf
            Loop Until ADF_tLock = True         ' Do it again if the PLL frequency is not locked
            DelayMS 10                          ' A small delay between frequency changes
        Next                                    ' Close the frequency scan loop
    Loop                                        ' Do it forever

'---------------------------------------------------------------------------
' Come here if the PLL lock timed out
' Transmit a message to a serial terminal
'
Sub PLL_TimeOut()
    HRSOutLn "PLL Lock Timed out"
EndSub

